#include "pappsomspp/core/types.h"
// Include the moc generated file because the Q_NAMESPACE-triggered static MetaObject
// needs to be exported to the users of this library.
#include "moc_types.cpp"

namespace pappso
{
std::map<Enums::DataKind, QString> dataKindMap{{Enums::DataKind::unset, "unset"},
                                               {Enums::DataKind::dt, "dt"},
                                               {Enums::DataKind::rt, "rt"},
                                               {Enums::DataKind::mz, "mz"}};
}
